

import csnd.*;

Csound               csoundInstance;
CsoundThread         csoundPerformanceThread = null;
SWIGTYPE_p_void      myvoid;
SWIGTYPE_p_CSOUND_   mycsound;
CsoundMYFLTArray     myfltarray;
float                myfloatpitch = 0;
float                myfloatamp = 0;

float xmag, ymag = 0;
float newXmag, newYmag = 0; 

void setup() 
{ 
  size(640, 360, P3D); 
  noStroke(); 
  colorMode(RGB, 1); 
  if ( csoundInstance == null ) 
  {
    csnd.csoundInitialize(null, null, csnd.CSOUNDINIT_NO_SIGNAL_HANDLER);
    csoundInstance = new Csound();
    myfltarray = new CsoundMYFLTArray();
    mycsound = csnd.csoundCreate(myvoid);
  }

  if ( csoundPerformanceThread != null ) 
  {
    if ( csoundPerformanceThread.isRunning() == true ) 
    {
      csoundInstance.Stop();
    }
  } 

  CsoundFile d = new CsoundFile(); 
  d.load("/home/JCH/sketchbook/CsndChngetRGBCube/chnget.csd");
  boolean b = d.exportForPerformance(); 
  int rc = csoundInstance.Compile("/home/JCH/sketchbook/CsndChngetRGBCube/chnget.csd");
  if ( rc == 0 ) 
  {
    CsoundThread  csoundPerformanceThread = new CsoundThread();
    csoundPerformanceThread.startCsound(); 
  }    
} 

void draw() 
{ 
  background(0.5);

  pushMatrix(); 

  translate(width/2, height/2, -30); 

  newXmag = mouseX/float(width) * TWO_PI;
  newYmag = mouseY/float(height) * TWO_PI;

  float diff = xmag-newXmag;
  if (abs(diff) >  0.01) { 
    xmag -= diff/4.0; 
  }

  diff = ymag-newYmag;
  if (abs(diff) >  0.01) { 
    ymag -= diff/4.0; 
  }

  rotateX(-ymag); 
  rotateY(-xmag); 

  myfloatpitch = xmag * 200;
  myfloatamp = (6.27 - ymag) * 4000;

  scale(90);
  beginShape(QUADS);

  fill(0, 1, .2); 
  vertex(-1,  1,  1);
  fill(0, 1, .2); 
  vertex( 1,  1,  1);
  fill(0, 1, .2); 
  vertex( 1, -1,  1);
  fill(0, 1, .2); 
  vertex(-1, -1,  1);

  fill(1, .1, 0); 
  vertex( 1,  1,  1);
  fill(1, .1, 0); 
  vertex( 1,  1, -1);
  fill(1, .1, 0); 
  vertex( 1, -1, -1);
  fill(1, .1, 0); 
  vertex( 1, -1,  1);

  fill(.1, 0, 1); 
  vertex( 1,  1, -1);
  fill(.1, 0, 1); 
  vertex(-1,  1, -1);
  fill(.1, 0, 1); 
  vertex(-1, -1, -1);
  fill(.1, 0, 1); 
  vertex( 1, -1, -1);

  fill(.7, 0, .7); 
  vertex(-1,  1, -1);
  fill(.7, 0, .7); 
  vertex(-1,  1,  1);
  fill(.7, 0, .7); 
  vertex(-1, -1,  1);
  fill(.7, 0, .7); 
  vertex(-1, -1, -1);

  fill(0, 1, .5); 
  vertex(-1,  1, -1);
  fill(0, 0, .5); 
  vertex( 1,  1, -1);
  fill(0, 0, .5); 
  vertex( 1,  1,  1);
  fill(0, 1, .5); 
  vertex(-1,  1,  1);

  fill(0, .2, .2); 
  vertex(-1, -1, -1);
  fill(1, .2, .2); 
  vertex( 1, -1, -1);
  fill(1, .2, .2); 
  vertex( 1, -1,  1);
  fill(0, .2, .2); 
  vertex(-1, -1,  1);

  endShape();

  popMatrix(); 
} 


class CsoundThread extends Thread 
{
  private boolean running = false;
  int x = 0;

  public void startCsound() 
  {
    running = true;
    start();
  }

  public void run() 
  {
    try {
      int myinputch = csndConstants.CSOUND_INPUT_CHANNEL;
      int mycontch = csndConstants.CSOUND_CONTROL_CHANNEL;
      SWIGTYPE_p_p_double myptr = myfltarray.GetPtr();

      while (csoundInstance.PerformKsmps() == 0) {   
        if (csnd.csoundGetChannelPtr(mycsound, myptr, "pitch", mycontch | myinputch) == 0)    
        {
          csoundInstance.SetChannel("pitch", myfloatpitch);
        } 
        if (csnd.csoundGetChannelPtr(mycsound, myptr, "amp", mycontch | myinputch) == 0)   
        {
          csoundInstance.SetChannel("amp", myfloatamp);
        }  
      } 
    } 
    catch (Exception e) {
    }
    csoundInstance.Stop();
    csoundInstance.Reset();
  } 

  public synchronized boolean isRunning() {
    return running;
  }
  public synchronized void  setRunning( boolean running ) {
    this.running = running;
  }
}


